package ij2x.plugin;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Color;
import java.awt.Dimension;

import java.util.MissingResourceException;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.gui.Roi;
import ij.plugin.PlugIn;

import ij2x.util.ResourceString;
import ij2x.util.BorderlessButton;
import ij2x.util.AWTImages;
import ij2x.util.ImageIcons;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2015</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class GelAnalyzerPanel extends JPanel implements PlugIn, ActionListener, MouseListener, MouseMotionListener {

    protected ImagePlus imp;
    protected ImageJ ij;
    public JButton infoButton;
    protected JButton labelPeakeButton, optionButton, plotLaneButton, profileoptionButton, plotprofileButton
                     , rePlotLaneButton, resetButton,  selectFirstLaneButton
                        , selectNextLaneButton, setRoiButton, undoButton, splineButton, bgsubtractorButton, scalebarButton, measureButton, measureOptionButton, measureSummarizeButton, measureClearButton, calibrationbarButton, removeoverlayButton;
    private JButton action;
    //new BevelBorder for LOWER or RAISED type
    private BevelBorder bb ;
    private Border lightGrayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.lightGray, Color.lightGray);
    private int actionCount = 0;
    private static int current;
    private long mouseDownTime;
    public static final int DOUBLE_CLICK_THRESHOLD = 150;
    public JPanel controlBarPanel=new JPanel();
    public JPanel panel=new JPanel();
    private GridBagLayout gblayout = new GridBagLayout();
    private static ResourceBundle resources;
    private int index = 0;
    private static boolean setSmooth;

    static {
        try {
            resources = ResourceBundle.getBundle("ij2x.plugin.resources.GelAnalyzer",
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("resources/GelAnalyzer.properties not found");
            System.exit(1);
        }
    }

    public GelAnalyzerPanel() {
    }
    public void run(String arg){
        ij = IJ.getInstance();
        controlBarPanel=createControlBar();
    }

    public JPanel createControlBar(){

        java.awt.FlowLayout flayout = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flayout.setHgap(0);
        flayout.setVgap(0);
        panel.setLayout(flayout);
        panel.setBorder(lightGrayBorder);

        ImageIcon b1o = (ImageIcon)ImageIcons.getIcons(this);

        AWTImages.setImageDir("gelanalyzerDir");


        String[] iconAllName = ResourceString.tokenize(ResourceString.getResourceString(resources, "gelanalyzerbar"));

        Image[][] icons = new Image[iconAllName.length][3];
        for(int i=0; i < iconAllName.length; i++){
            for(int j=0; j < 3; j++){
                icons[i][j]=((ImageIcon)AWTImages.createBufferedIcon(
                        this
                          , ResourceString.tokenize(ResourceString.getResourceString(resources, iconAllName[i]))[j]
                            , b1o.getImage()
                              , 24)).getImage();
            }
        }

        labelPeakeButton=BorderlessButton.createButton(this, labelPeakeButton, "Label Peake", icons[index++]);

        optionButton=BorderlessButton.createButton(this, optionButton, "Analayzer Option", icons[index++]);

        plotLaneButton=BorderlessButton.createButton(this, plotLaneButton, "Plot Lane", icons[index++]);

        profileoptionButton=BorderlessButton.createButton(this, profileoptionButton, "Profile Option", icons[index++]);

        plotprofileButton=BorderlessButton.createButton(this, plotprofileButton, "Plot Profile", icons[index++]);

        rePlotLaneButton=BorderlessButton.createButton(this, rePlotLaneButton, "Replot Lane", icons[index++]);

        resetButton=BorderlessButton.createButton(this, resetButton, "Reset Profile", icons[index++]);

        selectFirstLaneButton=BorderlessButton.createButton(this, selectFirstLaneButton, "Select First Lane", icons[index++]);

        selectNextLaneButton=BorderlessButton.createButton(this, selectNextLaneButton, "Select Next Lane", icons[index++]);

        splineButton=BorderlessButton.createButton(this, splineButton, "Set ROI", icons[index++]);

        removeoverlayButton=BorderlessButton.createButton(this, removeoverlayButton, "Remove Overlay", icons[index++]);
        
        undoButton=BorderlessButton.createButton(this, undoButton, "Undo Action", icons[index++]);
        
        measureButton=BorderlessButton.createButton(this, measureButton, "Measure", icons[index++]);
        
        measureSummarizeButton=BorderlessButton.createButton(this, measureSummarizeButton, "Summurize Measurements", icons[index++]);
        
        measureClearButton=BorderlessButton.createButton(this, measureClearButton, "Clear Results", icons[index++]);
        
        measureOptionButton=BorderlessButton.createButton(this, measureOptionButton, "Set Mesurements", icons[index++]);

        scalebarButton=BorderlessButton.createButton(this, scalebarButton, "Scale Bar", icons[index++]);
        
        calibrationbarButton=BorderlessButton.createButton(this, calibrationbarButton, "Calibration Bar", icons[index++]);
        
        infoButton=BorderlessButton.createButton(this, infoButton, "Show Info ...", icons[index]);

        //panel.add(bgsubtractorButton);
        panel.add(measureOptionButton);
        panel.add(scalebarButton);
        panel.add(calibrationbarButton);
        
        panel.add(removeoverlayButton);
        panel.add(undoButton);
        
        panel.add(measureButton);
        panel.add(measureOptionButton);
        panel.add(measureSummarizeButton);
        panel.add(measureClearButton);
        
        panel.add(splineButton);
        
        panel.add(selectFirstLaneButton);
        panel.add(selectNextLaneButton);
        panel.add(plotLaneButton);
        panel.add(rePlotLaneButton);
        panel.add(labelPeakeButton);
        panel.add(resetButton);
        panel.add(optionButton);
        
        panel.add(plotprofileButton);
        panel.add(profileoptionButton);
        
        panel.add(infoButton);
        
        return panel;
     }

     void setButtonDim(JButton action, Dimension maxDim, Dimension preDim, Dimension minDim){
       action.setMaximumSize(maxDim);
       action.setPreferredSize(preDim);
       action.setMinimumSize(minDim);
   }

     void setBorderlessButton(JButton button){
           button.setMargin(new Insets(0, 0, 0, 0));
           button.setBorderPainted(false);
           button.setContentAreaFilled(false);
           button.setFocusPainted(false);
       }

       public void mousePressed(MouseEvent e) {
           action = (JButton)e.getSource();
           int x = e.getX();
           boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
           mouseDownTime = System.currentTimeMillis();
                if (!doubleClick) {
                }
        }

        public void mouseReleased(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}

        public void mouseDragged(MouseEvent e) {}
        public void mouseMoved(MouseEvent e) {
            JButton action = (JButton)e.getSource();
            int x = e.getX();
            boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
            mouseDownTime = System.currentTimeMillis();
                 if (!false) {
                     if(action == infoButton){
                         IJ.showStatus("Command: Show Info...");
                     }
                     else if(action == labelPeakeButton){
                         IJ.showStatus("Label Peake");
                     }
                     else if(action == optionButton){
                         IJ.showStatus("Analayzer Option");
                     }
                     else if (action == plotLaneButton) {
                         IJ.showStatus("Plot Lane");
                     }
                     else if(action == profileoptionButton){
                         IJ.showStatus("Profile Option");
                     }
                     else if (action == plotprofileButton) {
                         IJ.showStatus("Plot Profile");
                     }
                     else if (action == rePlotLaneButton) {
                         IJ.showStatus("Replot Lane");
                     }
                     else if(action == resetButton){
                         IJ.showStatus("Reset Profile");
                     }
                     else if (action == selectFirstLaneButton) {
                         IJ.showStatus("Select First Lane");
                     }
                     else if(action == selectNextLaneButton){
                         IJ.showStatus("Select Next Lane");
                     }
                     else if(action == measureButton){
                         IJ.showStatus("Measure");
                     }
                     else if(action == measureOptionButton){
                         IJ.showStatus("Set Measurements");
                     }
                     else if(action == measureSummarizeButton){
                         IJ.showStatus("Summarize");
                     }
                     else if(action == measureClearButton){
                         IJ.showStatus("Clear Measure");
                     }
                     else if(action == scalebarButton){
                         IJ.showStatus("Scale Bar");
                     }
                     else if(action == calibrationbarButton){
                         IJ.showStatus("Calibration Bar");
                     }
                     //else if(action == bgsubtractorButton){
                     //    IJ.showStatus("Backgroen Subtractor");
                     //}
                     else if(action == removeoverlayButton){
                         IJ.showStatus("Remove Overlay");
                     }
                     else if(action == undoButton){
                         IJ.showStatus("Undo");
                     }
                     else if(action == splineButton){
                         IJ.showStatus("Set ROI");
                     }

                 }
        }
        
        public void actionPerformed(ActionEvent e){
            try {
                if(action == infoButton){
                    IJ.doCommand("Show Info...");
                }
                else if(action == labelPeakeButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "label");
                }
                else if(action == optionButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "options");
                }
                else if(action == plotLaneButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "plot");
                }
                else if(action == profileoptionButton){
                	//IJ.runPlugIn("ij.plugin.filter.Profiler", "set");
                	IJ.runPlugIn("ij.plugin.Profiler", "set");
                }
                else if (action == plotprofileButton) {
                	//IJ.runPlugIn("ij.plugin.filter.Profiler", "plot");
                	IJ.runPlugIn("ij.plugin.Profiler", "plot");
                }
                else if(action == rePlotLaneButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "replot");
                }
                else if(action == resetButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "reset");
                }
                else if(action == selectFirstLaneButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "first");
                }
                else if(action == selectNextLaneButton){
                    IJ.runPlugIn("ij.plugin.GelAnalyzer", "next");
                }
                else if(action == measureButton){
                    IJ.runPlugIn("ij.plugin.filter.Analyzer", "");
                }
                else if(action == measureOptionButton){
                    IJ.runPlugIn("ij.plugin.filter.Analyzer", "set");
                }
                else if(action == measureSummarizeButton){
                    IJ.runPlugIn("ij.plugin.filter.Analyzer", "sum");
                }
                else if(action == measureClearButton){
                    IJ.runPlugIn("ij.plugin.filter.Analyzer", "clear");
                }
                else if(action == scalebarButton){
                    IJ.runPlugIn("ij.plugin.ScaleBar", "");
                }
                else if(action == calibrationbarButton){
                    IJ.runPlugIn("ij.plugin.CalibrationBar", "");
                }
                //else if(action == bgsubtractorButton){
                    //IJ.runPlugIn("ij.plugin.filter.Filters", "smooth");
                    //IJ.runPlugIn("ij.plugin.filter.BackgroundSubtracter", "");
                //}
                else if(action == removeoverlayButton){
                    IJ.runPlugIn("ij.plugin.OverlayCommands", "remove");
                }
                else if(action == undoButton){
                    IJ.doCommand("Undo");
                }
                else if(action == splineButton){
                    //IJ.runPlugIn("ij2x.plugin.frame.HandleController", "");
                    //IJ.runPlugIn("ij2x.plugin.SpecifyROI", "drawroi");
                    IJ.runPlugIn("ij.plugin.SpecifyROI", "");
                }
            } catch (Exception ex) {

            }
        }
        void setSmooth(){
        	if(!setSmooth){
            IJ.runPlugIn("ij.plugin.filter.Filters", "smooth");
        	}
        }
}
